@@ -1,23 +1,24 @@
 """pyWorxCloud definition."""
 from __future__ import annotations
 
-import base64
-import contextlib
 import json
+import logging
 import sys
-import tempfile
 from datetime import datetime, timedelta
 from typing import Any
 
-import OpenSSL.crypto
-import paho.mqtt.client as mqtt
-from paho.mqtt.client import connack_string, error_string
-
 from .api import LandroidCloudAPI
 from .clouds import CloudType
 from .day_map import DAY_MAP
 from .events import EventHandler, LandroidEvent
-from .exceptions import AuthorizationError, MQTTException
+from .exceptions import (
+    AuthorizationError,
+    MowerNotFoundError,
+    NoOneTimeScheduleError,
+    NoPartymodeError,
+    OfflineError,
+    RequestException,
+)
 from .helpers import convert_to_time, get_logger
 from .utils import (
     MQTT,
@@ -30,11 +31,13 @@ from .utils import (
     Statistic,
     Weekdays,
 )
-from .utils.mqtt import PUBLISH_CALLS_LIMIT, PUBLISH_LIMIT_PERIOD, MQTTTopics
+from .utils.mqtt import Command
 from .utils.schedules import TYPE_TO_STRING
 
-if sys.version_info < (3, 9, 0):
-    sys.exit("The pyWorxcloud module requires Python 3.9.0 or later")
+if sys.version_info < (3, 10, 0):
+    sys.exit("The pyWorxcloud module requires Python 3.10.0 or later")
+
+_LOGGER = logging.getLogger(__name__)
 
 
 class WorxCloud(dict):
@@ -113,6 +116,9 @@ class WorxCloud(dict):
         Raise:
             TypeError: Error raised if invalid CloudType was specified.
         """
+        _LOGGER.debug("Initializing connector...")
+        super().__init__()
+
         self._worx_mqtt_client_id = None
 
         if not isinstance(
@@ -124,25 +130,33 @@ class WorxCloud(dict):
             ),
         ):
             try:
+                _LOGGER.debug("Try getting correct CloudType from %s", cloud.upper())
                 cloud = getattr(CloudType, cloud.upper())
+                _LOGGER.debug("Found cloud type %s", cloud)
             except AttributeError:
                 raise TypeError(
                     "Wrong type specified, valid types are: worx, landxcape or kress"
-                )
+                ) from None
 
+        _LOGGER.debug("Initializing the API connector ...")
         self._api = LandroidCloudAPI(username, password, cloud)
-
         self._username = username
         self._cloud = cloud
         self._auth_result = False
+        _LOGGER.debug("Getting logger ...")
         self._log = get_logger("pyworxcloud")
         self._raw = None
         self._tz = tz
 
         self._save_zones = None
         self._verify_ssl = verify_ssl
+        _LOGGER.debug("Initializing EventHandler ...")
         self._events = EventHandler()
 
+        self._endpoint = None
+        self._user_id = None
+        self._mowers = None
+
         # Dict of devices, identified by name
         self.devices = {}
 
@@ -152,8 +166,7 @@ class WorxCloud(dict):
         """Default actions using with statement."""
         self.authenticate()
 
-        self.connect(verify_ssl=self._verify_ssl)
-        self.update()
+        self.connect()
 
         return self
 
@@ -165,7 +178,8 @@ class WorxCloud(dict):
         """Authenticate against the API."""
         self._log.debug("Authenticating %s", self._username)
 
-        auth = self._authenticate()
+        self._api.get_token()
+        auth = self._api.authenticate()
         if auth is False:
             self._auth_result = False
             self._log.debug("Authentication for %s failed!", self._username)
@@ -202,13 +216,10 @@ class WorxCloud(dict):
 
     def disconnect(self) -> None:
         """Close API connections."""
-        if self.mqtt.connected:
-            self.mqtt.disconnect()
+        self.mqtt.disconnect()
 
     def connect(
         self,
-        verify_ssl: bool = True,
-        pahologger: bool = False,
     ) -> bool:
         """Connect to the cloud service endpoint
 
@@ -223,63 +234,24 @@ class WorxCloud(dict):
         self._fetch()
         self._log.debug("Done fetching basic API data")
 
+        self._endpoint = self._mowers[0]["mqtt_endpoint"]
+        self._user_id = self._mowers[0]["user_id"]
+
         self._log.debug("Setting up MQTT handler")
         # setup MQTT handler
         self.mqtt = MQTT(
-            self.devices,
-            self._worx_mqtt_client_id,
-            protocol=mqtt.MQTTv311,
+            self._api.access_token,
+            self._cloud.BRAND_PREFIX,
+            self._endpoint,
+            self._user_id,
+            self._on_update,
         )
 
-        if len(self.devices) > 3:
-            # Raising the ratelimit as we have more devices to handle
-            # !! This has potential to get you banned !!
-            self.mqtt.set_ratelimit(
-                PUBLISH_CALLS_LIMIT * len(self.devices), PUBLISH_LIMIT_PERIOD
-            )
+        for mower in self._mowers:
+            self.mqtt.subscribe(mower["mqtt_topics"]["command_out"])
 
-        self._log.debug("Done setting up MQTT handler, setting MQTT config.")
-
-        self.mqtt.endpoint = (
-            self._api._data[0]["mqtt_endpoint"]
-            if "mqtt_endpoint" in self._api._data[0]
-            else self._endpoint
-        )
-        self.mqtt.topics.update(
-            {
-                self._api._data[0]["name"]: MQTTTopics(
-                    self._api._data[0]["mqtt_topics"]["command_in"],
-                    self._api._data[0]["mqtt_topics"]["command_out"],
-                )
-            }
-        )
-        self.mqtt.reconnect_delay_set(60, 300)
-
-        self.mqtt.on_message = self._forward_on_message
-        self.mqtt.on_connect = self._on_connect
-        self.mqtt.on_disconnect = self._on_disconnect
-
-        if pahologger:
-            mqttlog = self._log.getChild("PahoMQTT")
-            self.mqtt.on_log = self._on_log
-            self.mqtt.enable_logger(mqttlog)
-            self.mqtt.logger = True
-        self._log.debug("Done setting MQTT config, fetching certificate.")
-
-        with self._get_cert() as cert:
-            self.mqtt.tls_set(certfile=cert)
-        self._log.debug("Done fetching certificate, setting TLS.")
-
-        if not verify_ssl:
-            self.mqtt.tls_insecure_set(True)
-
-        self._log.debug("Done setting TLS, beginning MQTT connect.")
-
-        self.mqtt.connect(self.mqtt.endpoint, port=8883, keepalive=600)
-        self._log.debug("MQTT connect done, starting loop")
-
-        self.mqtt.loop_start()
-        self._log.debug("MQTT loop started")
+        self.mqtt.connect()
+        self._log.debug("MQTT connect done")
 
         # Convert time strings to objects.
         self._log.debug("Converting date and time string")
@@ -296,72 +268,31 @@ class WorxCloud(dict):
         """Return current authentication result."""
         return self._auth_result
 
-    def _authenticate(self):
-        """Authenticate the user."""
-        auth_data = self._api.auth()
+    def _on_update(self, topic, payload, dup, qos, retain, **kwargs):
+        """Triggered when a MQTT message was received."""
+        data = json.loads(payload)
+        logger = self._log.getChild("MQTT_data_in")
+        logger.debug("MQTT data received '%s' on topic '%s'", payload, topic)
 
-        try:
-            self._api.set_token(
-                auth_data["access_token"],
-                auth_data["expires_in"],
-                auth_data["refresh_token"],
-            )
-            self._api.set_token_type(auth_data["token_type"])
-
-            self._api.get_profile()
-            profile = self._api.data
-            if profile is None:
-                return False
-            self._endpoint = profile["mqtt_endpoint"]
-            self._worx_mqtt_client_id = "android-" + self._api.uuid
-        except:  # pylint: disable=bare-except
-            return False
-
-    @contextlib.contextmanager
-    def _get_cert(self):
-        """Cet current certificate."""
-
-        certresp = self._api.get_cert()
-
-        with pfx_to_pem(certresp["pkcs12"]) as pem_cert:
-            yield pem_cert
-
-    def _forward_on_message(
-        self,
-        client: mqtt.Client | None,
-        userdata: Any | None,
-        message: Any | None,
-        properties: Any | None = None,  # pylint: disable=unused-argument
-    ) -> None:
-        """MQTT callback method definition."""
-        logger = self._log.getChild("mqtt.message_received")
-        topic = message.topic
-
-        for name, topics in self.mqtt.topics.items():
-            if topics["out"] == topic:
+        for mower in self._mowers:
+            if mower["serial_number"] == data["cfg"]["sn"]:
                 break
 
-        device: DeviceHandler = self.devices[name]
-
-        logger.debug(
-            "Received MQTT message for %s - processing data %s",
-            name,
-            message.payload.decode("utf-8"),
-        )
+        device: DeviceHandler = self.devices[mower["name"]]
 
         while not device.is_decoded:
-            pass  # Await last dataset to be handled before sending a new into the handler
+            pass  # Wait for last dataset to be handled
 
-        msg = message.payload.decode("utf-8")
-        if device.raw_data == msg:
+        if device.raw_data == data:
             self._log.debug("Data was already present and not changed.")
-            return  # Data was identical, update was not needed
+            return  # Dataset was not changed, no update needed
 
-        device.raw_data = msg
+        device.raw_data = payload
         self._decode_data(device)
-        device.capabilities.ready = True
 
-        self._events.call(LandroidEvent.DATA_RECEIVED, name=name, device=device)
+        self._events.call(
+            LandroidEvent.DATA_RECEIVED, name=mower["name"], device=device
+        )
 
     def _decode_data(self, device: DeviceHandler) -> None:
         """Decode incoming JSON data."""
@@ -381,8 +312,10 @@ class WorxCloud(dict):
             logger.debug("No valid data was found, skipping update for %s", device.name)
             return
 
+        # device.firmware["version"] = "{:.2f}".format(device.firmware["version"])
         if "dat" in data:
             device.rssi = data["dat"]["rsi"]
+            logger.debug("Status code: %s", data["dat"]["ls"])
             device.status.update(data["dat"]["ls"])
             device.error.update(data["dat"]["le"])
 
@@ -436,6 +369,7 @@ class WorxCloud(dict):
 
                 # Map current zone to zone index
                 device.zone.current = device.zone.indicies[device.zone.index]
+                # device.zone.current = 1
 
             # Fetch main schedule
             if "sc" in data["cfg"]:
@@ -557,134 +491,302 @@ class WorxCloud(dict):
         device.is_decoded = True
         logger.debug("Data for %s was decoded", device.name)
 
-    def _on_log(
-        self,
-        client: mqtt.Client | None,
-        userdata: Any | None,
-        level: Any | None,
-        buf: Any | None,
-    ) -> None:
-        """Capture MQTT log messages."""
-        logger = self._log.getChild("mqtt.log")
-        logger.debug("MQTT log message received: %s", buf)
-
-    def _on_connect(
-        self,
-        client: mqtt.Client | None,
-        userdata: Any | None,
-        flags: Any | None,
-        rc: int | None,
-        properties: Any | None = None,  # pylint: disable=unused-argument,invalid-name
-    ) -> None:
-        """MQTT callback method."""
-        logger = self._log.getChild("mqtt.connected")
-        logger.debug(connack_string(rc))
-        if rc == 0:
-            for name, topics in self.mqtt.topics.items():
-                topic = topics["out"]
-                logger.debug(
-                    "MQTT for %s connected, subscribing to topic '%s'", name, topic
-                )
-                client.subscribe(topic)
-
-            for name, device in self.devices.items():
-                device.mqtt = self.mqtt
-                if isinstance(device.raw_data, type(None)):
-                    logger.debug(
-                        "MQTT chached data not found for %s - requesting now", name
-                    )
-
-                    mqp = device.mqtt.send(name, force=True)
-                    if isinstance(mqp, type(None)):
-                        raise MQTTException("Couldn't send request to MQTT server.")
-                    elif not isinstance(mqp, str):
-                        while not mqp.is_published:
-                            pass
-
-            logger.debug("Setting MQTT connected flag TRUE")
-            self.mqtt.connected = True
-            self._events.call(LandroidEvent.MQTT_CONNECTION, state=self.mqtt.connected)
-
-        else:
-            logger.debug("Setting MQTT connected flag FALSE")
-            self.mqtt.connected = False
-            self._events.call(LandroidEvent.MQTT_CONNECTION, state=self.mqtt.connected)
-
-            raise MQTTException(connack_string(rc))
-
-    def _on_disconnect(
-        self,
-        client: mqtt.Client | None,
-        userdata: Any | None,
-        rc: int | None,
-        properties: Any | None = None,  # pylint: disable=unused-argument,invalid-name
-    ) -> None:
-        """MQTT callback method."""
-        logger = self._log.getChild("mqtt.disconnected")
-        if rc > 0:
-            if rc == 7:
-                if not self.mqtt.connected:
-                    raise MQTTException(
-                        "Unexpected MQTT disconnect - were you perhaps banned?"
-                    )
-
-            if self.mqtt.connected:
-                logger.debug("MQTT connection was lost! (%s)", error_string(rc))
-
-            logger.debug("Setting MQTT connected flag FALSE")
-            self.mqtt.connected = False
-            self._events.call(LandroidEvent.MQTT_CONNECTION, state=self.mqtt.connected)
-            for name, topics in self.mqtt.topics.items():
-                topic = topics["out"]
-                logger.debug(
-                    "MQTT for %s disconnected, unsubscribing from topic '%s'",
-                    name,
-                    topic,
-                )
-                client.unsubscribe(topic)
-
     def _fetch(self) -> None:
         """Fetch base API information."""
-        products = self._api.get_products()
+        self._mowers = self._api.get_mowers()
 
-        for product in products:
-            device = DeviceHandler(self._api, product)
-            self.devices.update({product["name"]: device})
+        for mower in self._mowers:
+            device = DeviceHandler(self._api, mower)
+            _LOGGER.debug("Mower '%s' data: %s", mower["name"], mower)
+            self.devices.update({mower["name"]: device})
 
-    def update(self) -> None:
-        """Retrive current device status."""
-        for name, device in self.devices.items():
-            status = self._api.get_status(device.serial_number)
-            status = str(status).replace("'", '"')
-
-            while not device.is_decoded:
-                pass  # Await previous dataset to be handled before sending a new into the handler.
-
-            device.raw_data = status
             self._decode_data(device)
 
+    def get_mower(self, serial_number: str) -> dict:
+        """Get a specific mower."""
+        for mower in self._mowers:
+            if mower["serial_number"] == serial_number:
+                return mower
 
-@contextlib.contextmanager
-def pfx_to_pem(pfx_data):
-    """Decrypts the .pfx file to be used with requests."""
-    with tempfile.NamedTemporaryFile(suffix=".pem", delete=False) as t_pem:
-        f_pem = open(t_pem.name, "wb")
-        p12 = OpenSSL.crypto.load_pkcs12(base64.b64decode(pfx_data), "")
-        f_pem.write(
-            OpenSSL.crypto.dump_privatekey(
-                OpenSSL.crypto.FILETYPE_PEM, p12.get_privatekey()
-            )
+        raise MowerNotFoundError(
+            f"Mower with serialnumber {serial_number} was not found."
         )
-        f_pem.write(
-            OpenSSL.crypto.dump_certificate(
-                OpenSSL.crypto.FILETYPE_PEM, p12.get_certificate()
+
+    def update(self, serial_number: str) -> None:
+        """Request a state refresh."""
+        mower = self.get_mower(serial_number)
+        _LOGGER.debug("Trying to refreshing '%s'", serial_number)
+        self.mqtt.ping(serial_number, mower["mqtt_topics"]["command_in"])
+
+    def start(self, serial_number: str) -> None:
+        """Start mowing task
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            _LOGGER.debug("Sending start command to '%s'", serial_number)
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.START
             )
-        )
-        certauth = p12.get_ca_certificates()
-        if certauth is not None:
-            for cert in certauth:
-                f_pem.write(
-                    OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, cert)
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def home(self, serial_number: str) -> None:
+        """Stop the current task and go home.
+        If the knifes was turned on when this is called,
+        it will return home with knifes still turned on.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+
+        if mower["online"]:
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.HOME
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def safehome(self, serial_number: str) -> None:
+        """Stop and go home with the blades off
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.SAFEHOME
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def pause(self, serial_number: str) -> None:
+        """Pause the mowing task
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.PAUSE
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def refresh(self, serial_number: str) -> None:
+        """Force a data refresh from API endpoint.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.FORCE_REFRESH
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def raindelay(self, serial_number: str, rain_delay: str) -> None:
+        """Set new rain delay.
+
+        Args:
+            rain_delay (str | int): Rain delay in minutes.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            if not isinstance(rain_delay, str):
+                rain_delay = str(rain_delay)
+            self.mqtt.publish(
+                serial_number,
+                mower["mqtt_topics"]["command_in"],
+                {"rd": rain_delay},
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def set_lock(self, serial_number: str, state: bool) -> None:
+        """Set the device locked state.
+
+        Args:
+            enabled (bool): True will lock the device, False will unlock the device.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            self.mqtt.command(
+                serial_number,
+                mower["mqtt_topics"]["command_in"],
+                Command.LOCK if state else Command.UNLOCK,
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def set_partymode(self, serial_number: str, state: bool) -> None:
+        """Turn on or off the partymode.
+
+        Args:
+            enable (bool): True is enabling partymode, Fasle is disabling partymode.
+
+        Raises:
+            NoPartymodeError: Raised if the device does not support partymode.
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            device = DeviceHandler(self._api, mower)
+            if device.capabilities.check(DeviceCapability.PARTY_MODE):
+                self.mqtt.publish(
+                    serial_number,
+                    mower["mqtt_topics"]["command_in"],
+                    {"sc": {"m": 2, "distm": 0}}
+                    if state
+                    else {"sc": {"m": 1, "distm": 0}},
                 )
-        f_pem.close()
-        yield t_pem.name
+            elif not device.capabilities.check(DeviceCapability.PARTY_MODE):
+                raise NoPartymodeError("This device does not support Partymode")
+        elif not mower["online"]:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def setzone(self, serial_number: str, zone: str) -> None:
+        """Set zone to be mowed when next mowing task is started.
+
+        Args:
+            zone (str | int): Zone to mow, valid possibilities are a number from 1 to 4.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            device = DeviceHandler(self._api, mower)
+            if not isinstance(zone, int):
+                zone = int(zone)
+
+            if device.zone["starting_point"][zone] == 0:
+                raise RequestException("Cannot request this zone as it is not defined.")
+
+            current = device.zone["indicies"]
+            new_zones = current
+
+            while not new_zones[device.zone["index"]] == zone:
+                tmp = []
+                for i in range(1, 10):
+                    tmp.append(new_zones[i])
+                tmp.append(new_zones[0])
+                new_zones = tmp
+
+            self.mqtt.publish(
+                serial_number,
+                mower["mqtt_topics"]["command_in"],
+                {"mzv": new_zones},
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def zonetraining(self, serial_number: str) -> None:
+        """Start the zone training task.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            _LOGGER.debug("Sending ZONETRAINING command to %s", mower["name"])
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.ZONETRAINING
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def restart(self, serial_number: str):
+        """Reboot the device baseboard.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            _LOGGER.debug("Sending RESTART command to %s", mower["name"])
+            self.mqtt.command(
+                serial_number, mower["mqtt_topics"]["command_in"], Command.RESTART
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def toggle_schedule(self, serial_number: str, enable: bool) -> None:
+        """Turn on or off the schedule.
+
+        Args:
+            enable (bool): True is enabling the schedule, Fasle is disabling the schedule.
+
+        Raises:
+            OfflineError: Raised if the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            self.mqtt.publish(
+                serial_number, mower["mqtt_topics"]["command_in"], {"sc": {"m": 1}}
+            ) if enable else str(
+                {"sc": {"m": 0}},
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def ots(self, serial_number: str, boundary: bool, runtime: str) -> None:
+        """Start a One-Time-Schedule task
+
+        Args:
+            boundary (bool): If True the device will start the task cutting the edge.
+            runtime (str | int): Minutes to run the task before returning to dock.
+
+        Raises:
+            NoOneTimeScheduleError: OTS is not supported by the device.
+            OfflineError: Raised when the device is offline.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            device = DeviceHandler(self._api, mower)
+            if device.capabilities.check(DeviceCapability.ONE_TIME_SCHEDULE):
+                if not isinstance(runtime, int):
+                    runtime = int(runtime)
+
+                self.mqtt.publish(
+                    serial_number,
+                    mower["mqtt_topics"]["command_in"],
+                    {"sc": {"ots": {"bc": int(boundary), "wtm": runtime}}},
+                )
+            elif not device.capabilities.check(DeviceCapability.ONE_TIME_SCHEDULE):
+                raise NoOneTimeScheduleError(
+                    "This device does not support Edgecut-on-demand"
+                )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
+
+    def send(self, serial_number: str, data: str) -> None:
+        """Send raw JSON data to the device.
+
+        Args:
+            data (str): Data to be sent, formatted as a valid JSON object.
+
+        Raises:
+            OfflineError: Raised if the device isn't online.
+        """
+        mower = self.get_mower(serial_number)
+        if mower["online"]:
+            _LOGGER.debug("Sending %s to %s", data, mower["name"])
+            self.mqtt.publish(
+                serial_number, mower["mqtt_topics"]["command_in"], json.loads(data)
+            )
+        else:
+            raise OfflineError("The device is currently offline, no action was sent.")
