@@ -6,12 +6,12 @@ from datetime import datetime
 from typing import Any
 from zoneinfo import ZoneInfo
 
-from pyworxcloud.utils.schedules import Schedule
+from ..utils.schedules import Schedule
 
-try:
-    from ..utils import __all__ as all_utils
-except:
-    pass
+# try:
+# from ..utils import __all__ as all_utils
+# except:
+#     pass
 
 DATE_FORMATS = [
     "%Y-%m-%d %H:%M:%S",
@@ -61,6 +61,8 @@ def convert_to_time(
     expression = (
         expression or r"\d{2,4}[-\/]\d{1,2}[-\/]\d{1,4} \d{1,2}:\d{1,2}:\d{1,2}"
     )
+    from ..utils import __all__ as all_utils
+
     if hasattr(data, "__dict__"):
         if isinstance(data, Schedule):
             pass
