@@ -46,9 +46,9 @@ class ProductInfo(LDict):
     ) -> None:
         """Get the device information based on ID."""
 
-        api_prod = None
-        if info_type == InfoType.MOWER:
-            api_prod = api.get_product_info(product_id)
+        # api_prod = None
+        # if info_type == InfoType.MOWER:
+        #     api_prod = api.get_product_info(product_id)
 
-        for attr, val in api_prod.items():
-            setattr(self, str(attr), val)
+        # for attr, val in api_prod.items():
+        #     setattr(self, str(attr), val)
